@@ -6,7 +6,48 @@ from sdata.timestamp import TimeStamp
 from sdata import __version__
 import json
 import os
+import hashlib
+import re
 
+def extract_name_unit(value):
+    """extract name and unit from a combined string
+
+    .. code-block:: python
+
+        value: 'Target Strain Rate (1/s) '
+        name : 'Target Strain Rate'
+        unit : '1/s'
+
+        value: 'Gauge Length [mm] monkey '
+        name : 'Gauge Length'
+        unit : 'mm'
+
+        value: 'Gauge Length <mm> whatever '
+        name : 'Gauge Length'
+        unit : 'mm'
+
+    :param value: string, e.g. 'Length <mm> whatever'
+    :return: name, unit
+    """
+    pattern1 = r'([\w\s\.]+) \(([\w.-\/]+)\)'
+    match1 = re.search(pattern1, value)
+    pattern2 = r'([\w\s\.]+) \[([\w.-\/]+)\]'
+    match2 = re.search(pattern2, value)
+    pattern3 = r'([\w\s\.]+) \<([\w.-\/]+)\>'
+    match3 = re.search(pattern3, value)
+    if match1:
+        name = match1.group(1)
+        unit = match1.group(2)
+    elif match2:
+        name = match2.group(1)
+        unit = match2.group(2)
+    elif match3:
+        name = match3.group(1)
+        unit = match3.group(2)
+    else:
+        name = value
+        unit = ""
+    return name, unit
 
 class Attribute(object):
     """Attribute class"""
@@ -61,15 +102,16 @@ class Attribute(object):
 
     def _set_value(self, value):
         try:
-            dtype = self.DTYPES.get(self.dtype, None)
-            if dtype is None:
-                dtype = self._guess_dtype(value)
-            self.dtype = dtype.__name__
-            if not value and self.dtype not in ["int", "float"]:
+            dtype = self.DTYPES.get(self.dtype, self._guess_dtype(value))
+
+            if self.dtype != dtype.__name__:
+                # logging.debug("guess dtype for ``: ``".format(value, dtype.__name__))
+                self.dtype = dtype.__name__
+            if not value and self.dtype not in ["int", "float", "bool"]:
                 self._value = None
             elif not value and self.dtype in ["int", "float"]:
                 self._value = np.nan
-            elif dtype.__name__ == "bool" and value in [0, "0", "False", "false"]:
+            elif dtype.__name__ == "bool" and value not in [1, "1", "true", "True"]:
                 self._value = False
             elif dtype.__name__ == "bool" and value in [1, "1", "true", "True"]:
                 self._value = True
@@ -180,6 +222,47 @@ s
     __repr__ = __str__
 
 
+class AttributeSchema(Attribute):
+    """Attribute class"""
+
+    DTYPES = {'float': float, 'int': int, 'str': str, 'timestamp': TimeStamp, "bool": bool}
+
+    def __init__(self, name, value, **kwargs):
+        Attribute.__init__(self, name, value, **kwargs)
+        self._required = False
+        self.required = kwargs.get("required", False)
+
+    def _get_required(self):
+        return self._required
+
+    def _set_required(self, value):
+        if value in [True, 1, "true", "True"]:
+            self._required = True
+        else:
+            self._required = False
+
+    required = property(fget=_get_required, fset=_set_required, doc="Attribute required")
+
+    def to_dict(self):
+        """:returns dict of attribute items"""
+        return {'name': self.name,
+                'value': self.value,
+                'unit': self.unit,
+                'dtype': self.dtype,
+                'description': self.description,
+                'label': self.label,
+                'required': self.required,
+                }
+
+    def to_list(self):
+        return [self.name, self.value, self.unit, self.dtype, self.description, self.label, self.required]
+
+
+    def __str__(self):
+        return "(AttrSchema'%s':%s(%s))" % (self.name, self.required, self.dtype)
+
+    __repr__ = __str__
+
 class Metadata(object):
     """Metadata container class
     
@@ -394,6 +477,21 @@ class Metadata(object):
         """
         self.set_attr(name, value, **kwargs)
 
+    def relabel(self, name, newname):
+        """relabel Attribute
+
+        :param name: old attribute name
+        :param newname: new attribute name
+        :return: None
+        """
+        attr = self.get(name)
+        if not attr:
+            logging.warning("{0}: no Attribute {1} to relabel.".format(self.__class__, name))
+        else:
+            attr.name = newname
+            self.attributes.pop(name)
+            self.add(attr)
+
     def get(self, name, default=None):
         if self._attributes.get(name) is not None:
             return self._attributes.get(name)
@@ -427,4 +525,82 @@ class Metadata(object):
         return len(self.attributes)
 
     def __getitem__(self, name):
-        return self.get(name)
\ No newline at end of file
+        return self.get(name)
+
+    def sort(self):
+        """sort Metadata.attributes
+
+        :return: None
+        """
+        sorted_keys = sorted(self.keys())
+        attributes = collections.OrderedDict([(key, self.attributes.get(key)) for key in sorted_keys])
+        self.attributes = attributes
+
+    @property
+    def sha3_256(self):
+        """Return a new SHA3 hash object with a hashbit length of 32 bytes.
+
+        :return: hashlib.sha3_256.hexdigest()
+        """
+        s = hashlib.sha3_256()
+        metadatastr = self.to_json().encode(errors="replace")
+        s.update(metadatastr)
+        return s.hexdigest()
+
+    def update_hash(self, hashobject):
+        """A hash represents the object used to calculate a checksum of a
+        string of information.
+
+        .. code-block:: python
+
+            hashobject = hashlib.sha3_256()
+            metadata = Metadata()
+            metadata.update_hash(hashobject)
+            hash.hexdigest()
+
+        :param hash: hash object
+        :return: hash_function().hexdigest()
+        """
+        if not (hasattr(hashobject, "update") and hasattr(hashobject, "hexdigest")):
+            logging.error("Metadata.hash: given hashfunction is invalid")
+            raise Exception("Metadata.hash: given hashfunction is invalid")
+
+        metadatastr = self.to_json().encode(errors="replace")
+        hashobject.update(metadatastr)
+        return hash
+
+
+class MetadataSchema(Metadata):
+    """MetadataSchema container class
+
+    each Metadata entry has has a
+        * name (256)
+        * value
+        * unit
+        * description
+        * type (int, str, float, bool, timestamp)
+        * label
+        * required
+        * min_value
+        * max_value
+        """
+
+    ATTRIBUTEKEYS = ["name", "value", "dtype", "unit", "description", "label", "required"]
+
+    def __init__(self, **kwargs):
+        """MetadataSchema class"""
+        Metadata.__init__(self, **kwargs)
+
+    def set_attr(self, name="N.N.", value=None, **kwargs):
+        """set AttributeSchema"""
+        if isinstance(name, AttributeSchema):
+            attr = name
+        else:
+            attr = self.get_attr(name) or AttributeSchema(name, value, **kwargs)
+        for key in ["dtype", "unit", "description", "label", "required"]:
+            if key in kwargs:
+                setattr(attr, key, kwargs.get(key))
+        if value is not None:
+            attr.value = value
+
+        self._attributes[attr.name] = attr
